package habibviewer.guielements;

/**
 * Class:           habibviewer.guielements.DecodingOptionsPanel
 * Version:         0.7
 * Date:            2004-01-18 16:14:56
 *
 * Description:     ?
 *
 * COPYRIGHT:
 *
 * This software module was originally developed by Pawe Haasa.
 *
 */

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.*;

public class DecodingOptionsPanel extends JPanel implements ActionListener, ItemListener, ChangeListener {
    private String bytes = "20000";
    private String bpp = "3.0";
    public int itemSelected = 0;
    private Checkbox noneCheckbox, byteCheckbox, bppCheckbox;
    private JSlider byteSlider, bppSlider;
    private TextField byteField, bppField;
    private Label byteLabel, bppLabel;
    public Button reloadButton;
    private Choice nrOfLayersChoice, nrOfBitPlanesChoice;

    public DecodingOptionsPanel() {
        setBorder(BorderFactory.createTitledBorder("Decoding options"));
        Box box = Box.createVerticalBox();
        add(box);

        Label decodingRateLabel = new Label("Decoding rate:");
        box.add(decodingRateLabel);

        CheckboxGroup cbg = new CheckboxGroup();
        noneCheckbox = new Checkbox("none", cbg, true);
        noneCheckbox.addItemListener(this);
        box.add(noneCheckbox);

        byteCheckbox = new Checkbox("in bytes", cbg, false);
        byteCheckbox.addItemListener(this);
        byteCheckbox.setEnabled(false);
        box.add(byteCheckbox);
        byteSlider = new JSlider(0, 20000, 20000);
        byteSlider.setEnabled(false);
        byteSlider.addChangeListener(this);
        box.add(byteSlider);
        Panel bytesBox = new Panel();
        box.add(bytesBox);
        byteField = new TextField(String.valueOf(byteSlider.getValue()), 7);
        byteField.setEnabled(false);
        byteField.addActionListener(this);
        bytesBox.add(byteField);
        byteLabel = new Label("bytes");
        byteLabel.setEnabled(false);
        bytesBox.add(byteLabel);

        bppCheckbox = new Checkbox("in bpp", cbg, false);
        bppCheckbox.addItemListener(this);
        bppCheckbox.setEnabled(false);
        box.add(bppCheckbox);
        bppSlider = new JSlider(0, 1000, 1000);
        bppSlider.setEnabled(false);
        bppSlider.addChangeListener(this);
        box.add(bppSlider);
        Panel bppBox = new Panel();
        box.add(bppBox);
        bppField = new TextField(String.valueOf(Math.floor(bppSlider.getValue())/100f), 4);
        bppField.setEnabled(false);
        bppField.addActionListener(this);
        bppBox.add(bppField);
        bppLabel = new Label("bpp");
        bppLabel.setEnabled(false);
        bppBox.add(bppLabel);

        Box layersBox = Box.createHorizontalBox();
        box.add(layersBox);
        Label nrOfLayersLabel = new Label("Number of layers");
        nrOfLayersLabel.setAlignment(Label.LEFT);
        layersBox.add(nrOfLayersLabel);
        nrOfLayersChoice = new Choice();
        nrOfLayersChoice.setEnabled(false);
        nrOfLayersChoice.add("1");
        layersBox.add(nrOfLayersChoice);

/*        Box cbBox = Box.createHorizontalBox();
        box.add(cbBox);
        Label nrOfCodeBlocks = new Label("Number of code blocks");
        nrOfCodeBlocks.setAlignment(Label.LEFT);
        cbBox.add(nrOfCodeBlocks);
        nrOfCodeBlocksChoice = new Choice();
        nrOfCodeBlocksChoice.setEnabled(false);
        for (int i = 1; i <= 32; i++)
            nrOfCodeBlocksChoice.add(Integer.toString(i));
        cbBox.add(nrOfCodeBlocksChoice);*/

        Box bpBox = Box.createHorizontalBox();
        box.add(bpBox);
        Label nrOfBitPlanesLabel = new Label("Number of bit planes");
        nrOfBitPlanesLabel.setAlignment(Label.LEFT);
        bpBox.add(nrOfBitPlanesLabel);
        nrOfBitPlanesChoice = new Choice();
        nrOfBitPlanesChoice.setEnabled(false);
        for (int i = 1; i <= 32; i++)
            nrOfBitPlanesChoice.add(Integer.toString(i));
        nrOfBitPlanesChoice.select(31);
        bpBox.add(nrOfBitPlanesChoice);

        box.add(Box.createVerticalStrut(5));
        reloadButton = new Button("Reload");
        reloadButton.setEnabled(false);
        reloadButton.addActionListener(this);
        box.add(reloadButton);
        validate();
    }

    public String getBytes() {
        try {
            int v = Integer.parseInt(byteField.getText());
            bytes = byteField.getText();
            byteSlider.setValue(v);
        } catch(Exception e) {
            byteField.setText(bytes);
        }
        repaint();
        validate();
        return bytes;
    }

    public String getBpp() {
        try {
            int v = Integer.parseInt(bppField.getText());
            bpp = bppField.getText();
            bppSlider.setValue(v);
        } catch(Exception e) {
            bppField.setText(bpp);
        }
        return bpp;
    }

    public int getNrOfLayers() {
        return nrOfLayersChoice.getSelectedIndex()+1;
    }

    public int getNrOfBitPlanes() {
        return nrOfBitPlanesChoice.getSelectedIndex()+1;
    }

    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source == byteField) {
            try {
                int v = Integer.parseInt(byteField.getText());
                if (v > byteSlider.getMaximum()) {
                    v = byteSlider.getMaximum();
                    byteField.setText(String.valueOf(v));
                } else if (v < 0) {
                    v = 0;
                    byteField.setText("0");
                }
                bytes = byteField.getText();
                byteSlider.setValue(v);
            } catch (Exception exc) {
                byteField.setText(String.valueOf(byteSlider.getValue()));
            }
        } else if (source == bppField) {
            try {
                float v = Float.parseFloat(bppField.getText());
                if (v > (float)bppSlider.getMaximum()/1000) {
                    v = (float)bppSlider.getMaximum()/1000;
                    bppField.setText(String.valueOf(v));
                } else if (v < 0) {
                    v = 0;
                    bppField.setText("0");
                }
                bpp = bppField.getText();
                bppSlider.setValue(Math.round(1000*v));
            } catch (Exception exc) {
                bppField.setText(String.valueOf(Math.floor(bppSlider.getValue())/1000f));
            }
        } else if (source == reloadButton) {
            if (itemSelected == 1) {
                actionPerformed(new ActionEvent(byteField, 0, ""));
            } else if (itemSelected == 2) {
                actionPerformed(new ActionEvent(bppField, 0, ""));
            }
        }
    }

    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();

        if (source == byteSlider) {
            bytes = String.valueOf(byteSlider.getValue());
            byteField.setText(bytes);
        } else if (source == bppSlider) {
            bpp = String.valueOf(bppSlider.getValue()/1000f);
            bppField.setText(bpp);
        }

        revalidate();
        repaint();
    }

    public void itemStateChanged(ItemEvent e) {
        Object source = e.getItemSelectable();

        if (source == noneCheckbox) {
            itemSelected = 0;
            setByteRateEnabled(false);
            setBppRateEnabled(false);
        } else if (source == byteCheckbox) {
            itemSelected = 1;
            setByteRateEnabled(true);
            setBppRateEnabled(false);
        } else if (source == bppCheckbox) {
            itemSelected = 2;
            setByteRateEnabled(false);
            setBppRateEnabled(true);
        }

        revalidate();
        repaint();
    }

    private void setByteRateEnabled(boolean b) {
        byteSlider.setEnabled(b);
        byteField.setEnabled(b);
        byteLabel.setEnabled(b);
        if (!b) byteField.setText(String.valueOf(byteSlider.getValue()));
    }

    private void setBppRateEnabled(boolean b) {
        bppSlider.setEnabled(b);
        bppField.setEnabled(b);
        bppLabel.setEnabled(b);
        if (!b) bppField.setText(String.valueOf(Math.floor(bppSlider.getValue())/1000f));
    }

    public void setEnabled(boolean b) {
        if (!b) {
            noneCheckbox.setState(true);
            setByteRateEnabled(false);
            setBppRateEnabled(false);
        }
        byteCheckbox.setEnabled(b);
        bppCheckbox.setEnabled(b);
        nrOfLayersChoice.setEnabled(b);
        nrOfBitPlanesChoice.setEnabled(b);
        reloadButton.setEnabled(b);
    }

    public void updateSettings(long fileSize, float bppRate, int numberOfLayers) {
        if (fileSize < Integer.MAX_VALUE)
            byteSlider.setMaximum((int)fileSize);
        else
            byteSlider.setMaximum(Integer.MAX_VALUE);

        bppSlider.setMaximum((int)(1000*bppRate));

        byteSlider.setValue(byteSlider.getMaximum());
        bytes = String.valueOf(byteSlider.getValue());
        byteField.setText(bytes);

        bppSlider.setValue(bppSlider.getMaximum());
        bpp = String.valueOf(bppSlider.getValue()/1000f);
        bppField.setText(bpp);

        nrOfLayersChoice.removeAll();
        for (int i = 1; i <= numberOfLayers; i++)
            nrOfLayersChoice.add(Integer.toString(i));
        nrOfLayersChoice.select(numberOfLayers-1);

        nrOfBitPlanesChoice.select(31);
    }
}
