package habibviewer.guielements;

/**
 * Class:           habibviewer.guielements.MainFrame
 * Version:         0.4
 * Date:            2003-12-17 03:17:39
 *
 * Description:     ?
 *
 * COPYRIGHT:
 *
 * This software module was originally developed by Pawe Haasa.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;

import habibviewer.guielements.filefilters.*;
import jj2000.j2k.image.BlkImgDataSrc;
import habibviewer.pgm.PGMImage;
import habibviewer.ppm.PPMImage;
import habibviewer.jpeg2000.Decoder;
import habibviewer.jpeg2000.Encoder;
import habibviewer.guielements.dialogs.AboutProgramDialog;
import habibviewer.guielements.dialogs.SaveAsJP2Dialog;

/**
 * This class creates main frame of program, whitch contains image area, menu
 * bar and settings panel.
 *
 * @see Frame
 * */
public class MainFrame extends Frame implements ActionListener, ItemListener {

    /** Area where the images are displayed */
    private ImageArea imageArea;

    /** Panel with file informations and progression settings. It's located
     * on the right */
    private SettingsPanel settingsPanel;

    BlkImgDataSrc theImage;
    private CheckboxMenuItem cmi;
    private MenuItem saveAs;
    private File currentFile = new File("");
    private String currentLoadDirectory = ".",
    currentSaveDirectory = ".";

    public MainFrame() {
        super("Habib JPEG2000 Viewer");
        setLayout(new BorderLayout());
        setSize(800, 680);
        setLocationRelativeTo(null);

        // MenuBar: File, View, Help
        MenuBar mb = new MenuBar();
        setMenuBar(mb);
        Menu file = new Menu("File");
        mb.add(file);
        file.add("Open...");
        saveAs = new MenuItem("Save as...");
        file.add(saveAs);
        saveAs.setEnabled(false);
        file.addSeparator();
        file.add("Properties");
        file.getItem(3).setEnabled(false);
        file.addSeparator();
        file.add("Exit");
        file.addActionListener(this);

        Menu view = new Menu("View");
        mb.add(view);
        cmi = new CheckboxMenuItem("Settings panel", true);
        view.add(cmi);
        cmi.addItemListener(this);
        view.addSeparator();
        view.add("Zoom in");
        view.add("Zoom out");
        view.addActionListener(this);

        Menu help = new Menu("Help");
        mb.add(help);
        help.add("About");
        help.addActionListener(this);

        // Settings panel
        settingsPanel = new SettingsPanel();
        settingsPanel.fileInfoPanel.chooseButton.addActionListener(this);
        settingsPanel.decodingOptionsPanel.reloadButton.addActionListener(this);
        add(settingsPanel, BorderLayout.EAST);

        // Image panel
        imageArea = new ImageArea();
        add(imageArea, BorderLayout.CENTER);

        setVisible(true);
        validate();

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                dispose();
                System.exit(0);
            }
        });
    }

    private void openJPEG2000(File file) {
        Decoder d;
        float rate;
        int nbytes;
        int layers, bitplanes, codeblocks;

        if (file == currentFile) {
            if (settingsPanel.decodingOptionsPanel.itemSelected == 1) {
                rate = -1;
                nbytes = Integer.parseInt(settingsPanel.decodingOptionsPanel.getBytes());
            } else if (settingsPanel.decodingOptionsPanel.itemSelected == 2) {
                rate = Float.parseFloat(settingsPanel.decodingOptionsPanel.getBpp());
                nbytes = -1;
            } else {
                rate = -1;
                nbytes = -1;
            }
            layers = settingsPanel.decodingOptionsPanel.getNrOfLayers();
            bitplanes = settingsPanel.decodingOptionsPanel.getNrOfBitPlanes();
        } else {
            rate = -1;
            nbytes = -1;
            layers = -1;
            codeblocks = -1;
            bitplanes = -1;
        }

        codeblocks = -1;
        d = new Decoder(imageArea, file.getPath(), rate, nbytes,
                layers, bitplanes, codeblocks);
        Thread thread = new Thread(d);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            printError("Exception: Thread interrupted");
        }

        if (d.getExitCode() != 0)
            printError(d.errorMessage);
        else {
            theImage = d.decodedImage;
            if (file != currentFile) {
                currentFile = file;
                String addInfo =
                        "Number of layers: " + d.nLayers + "\n" +
                        "Number of tiles: " + d.nTiles + "\n" +
                        "Tiles size: " + d.tileWidth + " x " + d.tileHeight + "\n" +
                        "Number of components in codestream: " + d.nCompCod + "\n" +
                        "Reconstructed resolution: " + d.reconstructedImageWidth + " x " +
                        d.reconstructedImageHeight + "\n" +
                        "Target resolution: " + d.targetImageWidth + " x " +
                        d.targetImageHeight + "\n" +
                        "Reconstructed resolution level: " + d.reconstructedResolutionLevel + "\n" +
                        "Target resolution level: " + d.targetResolutionLevel + "\n" +
                        "Target bitrate: " + (float)Math.round(d.targetBitrate*1000)/1000 + "\n";
                String COMInfo[] = d.getCOMInfo();
                for (int i = 0; i < COMInfo.length; ++i)
                    addInfo += COMInfo[i] + "\n";

                settingsPanel.fileInfoPanel.updateInfo(file.getPath(), file.length(),
                        d.targetImageWidth, d.targetImageHeight, (float) file.length() * 8 /
                        (d.targetImageWidth * d.targetImageHeight), addInfo);
                settingsPanel.decodingOptionsPanel.setEnabled(true);
                settingsPanel.decodingOptionsPanel.updateSettings(file.length(),
                        (float) file.length() * 8 / (imageArea.getImage().getWidth(imageArea)
                        * imageArea.getImage().getHeight(imageArea)), d.nLayers);

                saveAs.setEnabled(true);
            }
        }
    }

    private void openPGM(File file) {
        Image image = PGMImage.load(file.getPath());
        if (image == null) {
            printError("Incorrect file format!");
            return;
        }
        currentFile = file;
        imageArea.setImage(image);
        String info = "PGM Image";
        settingsPanel.fileInfoPanel.updateInfo(file.getPath(), file.length(), 0, 0, 0, info);
        settingsPanel.decodingOptionsPanel.setEnabled(false);
        saveAs.setEnabled(true);
    }

    private void openPPM(File file) {
        Image image = PPMImage.load(file.getPath());
        if (image == null) {
            printError("Incorrect file format!");
            return;
        }
        currentFile = file;
        imageArea.setImage(image);
        String info = "PPM Image";
        settingsPanel.fileInfoPanel.updateInfo(file.getPath(), file.length(), 0, 0, 0, info);
        settingsPanel.decodingOptionsPanel.setEnabled(false);
        saveAs.setEnabled(true);
    }

    private void openStandardImage(File file) {
        Image image = Toolkit.getDefaultToolkit().getImage(file.getPath());
        prepareImage(image, imageArea);
        imageArea.setImage(image);
        if (image.getWidth(null) == 0) {
            printError("Incorrect file format!");
            return;
        }
        currentFile = file;
        String info = "";
        settingsPanel.fileInfoPanel.updateInfo(file.getPath(), file.length(), 0, 0, 0, info);
        settingsPanel.decodingOptionsPanel.setEnabled(false);
        saveAs.setEnabled(true);
    }

    private void saveJPEG2000(File file) {
        SaveAsJP2Dialog saveDialog = new SaveAsJP2Dialog(this, file);
        if (saveDialog.returnVal != SaveAsJP2Dialog.CONFIRM_OPTION) return;

        Encoder e = new Encoder(theImage, file.getPath(), saveDialog.bitrate,
                saveDialog.numberOfLayers);
        Thread thread = new Thread(e);
        thread.start();
        try {
            thread.join();
        } catch (Exception ex) {
        }
        if (e.exitCode != 0)
            printError(e.errorMessage);
        else
            openJPEG2000(file);
    }

    private void savePGM(File file) {
        PGMImage image = new PGMImage(imageArea.getImage());
        if (!image.save(file.getPath())) {
            printError("Error while saving file " + file.getName());
            return;
        }
    }

    private void savePPM(File file) {
        PPMImage image = new PPMImage(imageArea.getImage());
        if (!image.save(file.getPath())) {
            printError("Error while saving file " + file.getName());
            return;
        }
    }

    private void openFileDialog() {
        JFileChooser chooser = new JFileChooser(currentLoadDirectory);
        GIFFileFilter gifFilter = new GIFFileFilter();
        JPEGFileFilter jpegFilter = new JPEGFileFilter();
        JPEG2000FileFilter jpeg2000Filter = new JPEG2000FileFilter();
        PNGFileFilter pngFilter = new PNGFileFilter();
        PGMFileFilter pgmFilter = new PGMFileFilter();
        PPMFileFilter ppmFilter = new PPMFileFilter();
        chooser.setFileFilter(jpeg2000Filter);

        chooser.setFileFilter(gifFilter);
        chooser.setFileFilter(jpegFilter);
        chooser.setFileFilter(pngFilter);
        chooser.setFileFilter(pgmFilter);
        chooser.setFileFilter(ppmFilter);
        chooser.setFileFilter(new AllImagesFileFilter());

        chooser.setAcceptAllFileFilterUsed(false);
        int returnVal = chooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = chooser.getSelectedFile();
            if (!file.exists()) {
                JOptionPane.showMessageDialog(this, "Cannot find file " + file.getPath(),
                        "Loading Error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            currentLoadDirectory = file.getParent();
            if (jpeg2000Filter.accept(file)) {
                openJPEG2000(file);
            } else if (pgmFilter.accept(file)) {
                openPGM(file);
            } else if (ppmFilter.accept(file)) {
                openPPM(file);
            } else if (jpegFilter.accept(file)) {
                openStandardImage(file);
            } else if (gifFilter.accept(file)) {
                openStandardImage(file);
            } else if (pngFilter.accept(file)) {
                openStandardImage(file);
            }
        }
    }

    private void saveFileDialog() {
        JFileChooser chooser = new JFileChooser(currentSaveDirectory);
        chooser.setSelectedFile(new File(currentSaveDirectory + "\\" +
                currentFile.getName().substring(0, currentFile.getName().lastIndexOf(".")) + ".JP2"));
        PGMFileFilter pgmFilter = new PGMFileFilter();
        PPMFileFilter ppmFilter = new PPMFileFilter();
        JPEG2000FileFilter jp2Filter = new JPEG2000FileFilter();
        chooser.setFileFilter(pgmFilter);
        chooser.setFileFilter(ppmFilter);
        chooser.setFileFilter(jp2Filter);
        chooser.setAcceptAllFileFilterUsed(false);
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = chooser.getSelectedFile();
            currentSaveDirectory = file.getParent();
            if (pgmFilter.accept(file)) {
                savePGM(file);
            } else if (ppmFilter.accept(file)) {
                savePPM(file);
            } else if (jp2Filter.accept(file)) {
                saveJPEG2000(file);
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        String ac = e.getActionCommand();
        Object source = e.getSource();

        if ("Open...".equals(ac)) {
            openFileDialog();
        } else if ("Save as...".equals(ac)) {
            saveFileDialog();
        } else if ("Exit".equals(ac)) {
            dispose();
            System.exit(0);
        } else if ("Zoom in".equals(ac)) {
            imageArea.zoomIn();
        } else if ("Zoom out".equals(ac)) {
            imageArea.zoomOut();
        } else if ("About".equals(ac)) {
            new AboutProgramDialog(this);
        }

        if (source == settingsPanel.fileInfoPanel.chooseButton) {
            openFileDialog();
        } else if (source == settingsPanel.decodingOptionsPanel.reloadButton) {
            openJPEG2000(currentFile);
        }
    }

    public void itemStateChanged(ItemEvent e) {
        Object source = e.getSource();

        if (source == cmi) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                settingsPanel.setVisible(true);
            } else {
                settingsPanel.setVisible(false);
            }
        }
        validate();
    }

    public void printError(String msg) {
        JOptionPane.showMessageDialog(this, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }
}
