/**
 * Class:           habibviewer.guielements.dialogs.SaveAsJP2Dialog
 * Version:         0.5
 * Date:            2004-01-18 16:16:09
 *
 * Description:     ?
 *
 * COPYRIGHT:
 *
 * This software module was originally developed by Pawe Haasa.
 *
 */

package habibviewer.guielements.dialogs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

public class SaveAsJP2Dialog extends Dialog implements ActionListener, ItemListener {
    private Button confirmButton, cancelButton;
    private Checkbox losslessCheckbox, lossyCheckbox, bitrateCheckbox;
    private TextField bitrateField, layersField;
    private Label bitrateLabel, layersLabel;

    static final public int CONFIRM_OPTION = 1;
    static final public int CANCEL_OPTION = 2;
    public int returnVal = CANCEL_OPTION;
    public float bitrate = 0.5f;
    public int numberOfLayers = 5;

    public SaveAsJP2Dialog(Frame parent, File file) {
        super(parent, file.getName() + " save options", true);
        setSize(300, 220);
        setLocationRelativeTo(parent);
        setResizable(false);

        JPanel p1 = new JPanel();
        p1.setBorder(BorderFactory.createTitledBorder("Compression options"));
        p1.setAlignmentX(JPanel.LEFT_ALIGNMENT);
        add(p1);

        Box box = Box.createVerticalBox();
        p1.add(box);

        CheckboxGroup cbg = new CheckboxGroup();
        losslessCheckbox = new Checkbox("Lossless compression", cbg, true);
        losslessCheckbox.addItemListener(this);
        box.add(losslessCheckbox);

        lossyCheckbox = new Checkbox("Lossy compression", cbg, false);
        lossyCheckbox.addItemListener(this);
        box.add(lossyCheckbox);

        // bitrate box contains: [x] Output bitrate: [  n] bpp
        Box bitrateBox = Box.createHorizontalBox();
        box.add(bitrateBox);
        bitrateCheckbox = new Checkbox("Output bitrate :  ");
        bitrateCheckbox.addItemListener(this);
        bitrateCheckbox.setEnabled(false);
        bitrateBox.add(bitrateCheckbox);
        bitrateField = new TextField(String.valueOf(bitrate), 5);
        bitrateField.addActionListener(this);
        bitrateField.setEnabled(false);
        bitrateBox.add(bitrateField);
        bitrateLabel = new Label("bpp");
        bitrateLabel.setEnabled(false);
        bitrateBox.add(bitrateLabel);
        box.add(Box.createVerticalStrut(10));

        // layers box contains: Number of layers: [  n]
        Box layersBox = Box.createHorizontalBox();
        box.add(layersBox);
        layersLabel = new Label("Number of layers : ");
        layersBox.add(layersLabel);
        layersField = new TextField(String.valueOf(numberOfLayers), 5);
        layersField.addActionListener(this);
        layersBox.add(layersField);
        box.add(Box.createVerticalStrut(10));

        Box buttonBox = Box.createHorizontalBox();
        buttonBox.setAlignmentX(Box.CENTER_ALIGNMENT);
        box.add(buttonBox);
        confirmButton = new Button("      OK      ");
        confirmButton.addActionListener(this);
        buttonBox.add(confirmButton);
        buttonBox.add(Box.createHorizontalStrut(10));
        cancelButton = new Button("    Cancel    ");
        cancelButton.addActionListener(this);
        buttonBox.add(cancelButton);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                dispose();
            }
        });

        show();
        validate();
    }

    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source == confirmButton) {
            if (lossyCheckbox.getState()) {
                if (bitrateCheckbox.getState()) {
                    try {
                        bitrate = Float.parseFloat(bitrateField.getText());
                        if (bitrate <= 0) bitrate = 0.001f;
                    } catch (Exception ex) {
                    }
                } else
                    bitrate = -1;
            } else
                bitrate = 0;
            try {
                numberOfLayers = Integer.parseInt(layersField.getText());
                if (numberOfLayers <= 0) numberOfLayers = 1;
            } catch (Exception ex) {
            }
            returnVal = CONFIRM_OPTION;
            dispose();
        } else if (source == cancelButton) {
            returnVal = CANCEL_OPTION;
            dispose();
        } else if (source == bitrateField) {
            try {
                bitrate = Float.parseFloat(bitrateField.getText());
                if (bitrate <= 0) bitrate = 0.001f;
            } catch (Exception ex) {
            }
            bitrateField.setText(String.valueOf(bitrate));
        } else if (source == layersField) {
            try {
                numberOfLayers = Integer.parseInt(layersField.getText());
                if (numberOfLayers <= 0) numberOfLayers = 1;
            } catch (Exception ex) {
            }
            layersField.setText(String.valueOf(numberOfLayers));
        }
    }

    private void setLossyOptionsEnabled(boolean b) {
        bitrateCheckbox.setEnabled(b);
        bitrateField.setEnabled(b & bitrateCheckbox.getState());
        bitrateLabel.setEnabled(b & bitrateCheckbox.getState());
    }

    public void itemStateChanged(ItemEvent e) {
        Object source = e.getItemSelectable();

        if (source == losslessCheckbox) {
            bitrate = 0;
            setLossyOptionsEnabled(false);
        } else if (source == lossyCheckbox) {
            try {
                bitrate = Float.parseFloat(bitrateField.getText());
                if (bitrate <= 0) bitrate = 0.001f;
            } catch (Exception ex) {
            }
            bitrateField.setText(String.valueOf(bitrate));
            setLossyOptionsEnabled(true);
        } else if (source == bitrateCheckbox) {
            setLossyOptionsEnabled(true);
        }
    }
}
