package habibviewer.ppm;

/**
 * Class:           habibviewer.ppm.PPMImage
 * Version:         0.?
 * Date:            2004-01-21 01:20:30
 *
 * Description:     ?
 *
 * COPYRIGHT:
 *
 * This software module was originally developed by Pawe? Ha?asa.
 * 
 */

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.lang.String;

public class PPMImage implements ImageConsumer {
    private int wys, szer;
    private ImageProducer producer;
    private byte[] Data = null;
    private boolean flag = false;

    public PPMImage(Image img) {
        producer = img.getSource();
    }

    public static Image load(String PPMfile) {
        File f = new File(PPMfile);
        FileInputStream in;

        Image img = null;
        int xres = 0, yres = 0, maxValue = 0;
        try {
            in = new FileInputStream(f);
        } catch (IOException e) {
            return null;
        }
        try {
            StreamTokenizer st = new StreamTokenizer(new InputStreamReader(in));
            st.commentChar('#');
            st.eolIsSignificant(true);
            st.nextToken();
            if (!(st.sval.equals("P6"))) {
                // Unsupported PPM version
                return null;
            }
            if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                xres = (int) st.nval;
            else {
                while (st.nextToken() != StreamTokenizer.TT_NUMBER) {
                }
                xres = (int) st.nval;
            }
            if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                yres = (int) st.nval;
            else {
                while (st.nextToken() != StreamTokenizer.TT_NUMBER) {
                }
                yres = (int) st.nval;
            }
            if (st.nextToken() == StreamTokenizer.TT_NUMBER)
                maxValue = (int) st.nval;
            else {
                while (st.nextToken() != StreamTokenizer.TT_NUMBER) {
                }
                maxValue = (int) st.nval;
            }

            while (st.nextToken() != StreamTokenizer.TT_EOL) {
            }
            byte data[] = new byte[xres * yres * 3];	// pixel = 3bytes
            in.read(data);
            int obraz[] = new int[xres * yres];

            for (int i = 0; i < yres; i++) {
                int k = 0;
                for (int j = 0; j < 3 * xres; j += 3) {
                    int r = (-(data[i * 3 * xres + j] + 1) << 16) & 0x00ff0000;
                    int g = (-(data[i * 3 * xres + j + 1] + 1) << 8) & 0x0000ff00;
                    int b = (-(data[i * 3 * xres + j + 2])) & 0x000000ff;

                    obraz[i * xres + k] = -r - g - b;
                    k++;
                }
            }
            img = (new Canvas()).createImage(new MemoryImageSource(xres, yres, obraz, 0, xres));
        } catch (Exception e) {
            return null;
        }
        return img;
    }

    public boolean save(String PPMfile) {
        producer.startProduction(this);
        while (!this.flag) {
            try {
                Thread.sleep(200);
            } catch (InterruptedException ie) {
            }
        }
        File f = new File(PPMfile);
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(f);
        } catch (IOException e) {
            return false;
        }
        try {
            out.write("P6\n#Created by ImageViewer\n".getBytes());
            out.write(String.valueOf(szer).getBytes());
            out.write(" ".getBytes());
            out.write(String.valueOf(wys).getBytes());
            out.write("\n255\n".getBytes());
            out.write(this.Data);
            out.close();
        } catch (IOException e) {
            try {
                out.close();
            } catch (IOException ioe) {
            }
            return false;
        }
        return true;
    }

    public void setDimensions(int width, int height) {
        wys = height;
        szer = width;
        Data = new byte[wys * szer * 3];
    }

    public void setProperties(java.util.Hashtable props) {
    }

    public void setColorModel(ColorModel model) {
    }

    public void setHints(int hintflag) {
    }

    public void setPixels(int x, int y, int w, int h, ColorModel model, byte pixels[], int off, int scansize) {
        for (int i = 0; i < h; i++) {
            int k = 0;
            for (int j = 0; j < w; j++) {
                int pix = (int) (pixels[i * scansize + j + off]);
                Data[(y + i) * 3 * this.szer + x + k] = (byte) model.getRed(pix);
                Data[(y + i) * 3 * this.szer + x + k + 1] = (byte) model.getGreen(pix);
                Data[(y + i) * 3 * this.szer + x + k + 2] = (byte) model.getBlue(pix);
                k += 3;
            }
        }
    }

    public void setPixels(int x, int y, int w, int h, ColorModel model, int pixels[], int off, int scansize) {
        for (int i = 0; i < h; i++) {
            int k = 0;
            for (int j = 0; j < w; j++) {
                int pix = pixels[i * scansize + j + off];
                Data[(y + i) * 3 * this.szer + x + k] = (byte) model.getRed(pix);
                Data[(y + i) * 3 * this.szer + x + k + 1] = (byte) model.getGreen(pix);
                Data[(y + i) * 3 * this.szer + x + k + 2] = (byte) model.getBlue(pix);
                k += 3;
            }
        }
    }

    public void imageComplete(int status) {
        switch (status) {
            case IMAGEABORTED:
                flag = true;
                break;
            case STATICIMAGEDONE:
                flag = true;
                break;
            case SINGLEFRAMEDONE:
                flag = true;
                break;
            case IMAGEERROR:
                flag = true;
                break;
            default:
                break;
        }
        producer.removeConsumer(this);
    }
}

